#!/usr/bin/perl -wT
#
# admin/gmod.cgi
#
# Modifies a group entry in the /etc/groups file.
#
# Copyright 1998 Corel Computer Corporation
#
# This is open-source software (see www.opensource.org).  Open Source(R)
# is a Registered Certification Mark of Software in the Public Interest.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as 
# published by the Free Software Foundation; either version 1,
# or, (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to the
# Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

use strict;
use CGI;
use CGI::Carp qw/fatalsToBrowser/;
my $q = new CGI;
require '../layout.pl';
require '../common.pl';

BEGIN { $ENV{'PATH'} = '/bin:/usr/bin'; }
my %system_groups = read_group();
my @key = keys %system_groups;

# Any errors found during processing added here:

my @errors;

# Begin validation sequence

push @errors, "Remote username could not be determined."
unless defined $ENV{'REMOTE_USER'};

push @errors, "The name must contain only alphanumeric characters"
if $q->param('group')  =~ /[^-_.a-zA-Z0-9]/;

if ( $q->param('newgroup') eq 1 )
{
  for ( my $count = 0 ; $count <= $#key ; $count++ )
  {
    my $current_group = $key[$count];
    if ($current_group eq $q->param('group'))
    {
      push @errors, "There is already a group called $current_group on the system."
    }
  }
}


# Check for errors, abort if any found.

if (@errors) {
  print start_webfront("Confirmation","admin",3,-1);
  print CGI::h1((($#errors > 0) ? "Errors have" : "An error has")." been detected");
  print CGI::ul(CGI::li(\@errors));
  print CGI::p(CGI::i('Go <A HREF="grouplist.cgi">back</A> and try again.'));
  print end_webfront();
  exit(1);
} 

# Do regular processing
my @gparm = ('','','','');
my $replacement = '';
$gparm[0] = $q->param('group');

if ( $q->param('newgroup') eq 1 )
{
  # Create a new group
  $gparm[1] = 'x';
  $gparm[2] = $q->param('gid');
  $replacement = join(":", @gparm[0,1,2]) . ":\n";
}
else 
{
  # Check for the existance of a passwd for this group
  my $field = $system_groups{ $gparm[0] } ;
  $gparm[1] = $$field[1]; 
 
  # Assign the gid
  $gparm[2] = $$field[2];

  # Process the users
  my @members = split(/,/, $$field[3]) ;
  if (  $q->param('status') eq 'remove' )
  {
    #remove the user;
    my @newmembers = ();
    
    for ( my $countusers = 0 ; $countusers <= $#members ; $countusers++ )
    {
      if ( $q->param('user') ne $members[$countusers] )
      {
        $newmembers[$#newmembers+1] = $members[$countusers];
      }
    }
    @members = @newmembers ;
  }
  else
  {           #####add users
    foreach my $i ($q->param('user'))
      {
        push @members, $i
        unless $i eq 'none';
      }
  }
  $gparm[3] = join(",", @members) ;
  $replacement = join(":", @gparm[0,1,2,3]) . "\n";
}

# Now scan/replace group file entry
umask(022);
open IN, "/etc/group" or die "Can't read group\n";
open OUT, ">/etc/group.new" or die "Can't write group\n";
while (my $line = <IN>) {
  my @line = split /:/, $line;
  # Only change lines with matching group, and GID>100
  if ($line[0] eq $gparm[0] and $line[2] >= 100) {
    print OUT $replacement;
  } else {
    print OUT $line;
  }
}
# Add new groups to the system
if ( $q->param('newgroup') eq 1 )
{
  print OUT $replacement;
}
close OUT;
close IN;
system "mv /etc/group.new /etc/group";

print start_webfront("Confirmation","admin",3,-1);
print CGI::h1("Operation was successful.");
print CGI::p('Click <A HREF="grouplist.cgi">here</A> to return.');
print end_webfront();
