#!/usr/bin/perl -wT
#
# admin/umod.cgi
#
# Modifiy a user (name, password, shell, etc).
#
# Copyright 1998 Corel Computer Corporation
#
# This is open-source software (see www.opensource.org).  Open Source(R)
# is a Registered Certification Mark of Software in the Public Interest.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as 
# published by the Free Software Foundation; either version 1,
# or, (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to the
# Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

use strict;
use CGI;
use CGI::Carp qw/fatalsToBrowser/;
my $q = new CGI;
require '../layout.pl';
require '../common.pl';
BEGIN { $ENV{'PATH'} = '/bin:/usr/bin'; }

# Any errors found during processing added here:

my @errors;

# Begin validation sequence

push @errors, "Remote username could not be determined."
unless defined $ENV{'REMOTE_USER'};

push @errors, "You must enter a real name."
unless defined $q->param('nam') and $q->param('nam') ne '';
my $name = $q->param('nam');

push @errors, "The name must contain only alphanumeric characters"
if $name =~ /[^-_. a-zA-Z0-9]/;

my $login = $q->param('login');
push @errors, "The login must contain only alphanumeric characters"
if $login =~ /[^-_.a-zA-Z0-9]/;

my @user = getpwnam($login);

my $newpass;
if (defined $q->param('pw1') and defined $q->param('pw2')) {
  push @errors, "Passwords do not match, please try again."
  unless $q->param('pw1') eq $q->param('pw2');
  $newpass = $q->param('pw1');
}

if (( defined $q->param('pw1')) && ($q->param('pw1') eq '') && ($q->param('newuser') eq 1))
{
  push @errors, "You must set the password for a new user."; 
}

if ( $q->param('newuser') eq 1 )
{
  my %system_users = read_passwd();
  my @key = keys %system_users;
  for ( my $count = 0 ; $count <= $#key ; $count++ )
  {
    my $current_user = $key[$count];
    if ($current_user eq $login)
    {
      push @errors, "There is already a user `$login' on the system."
    }
  }
  # Ensure that reserved names (root, bin, ...) are not used:
  if ($login eq $user[0]) {
    push @errors, "The login name `$login' is reserved and can not be used.";
  }
}


# Check for errors, abort if any found.

if (@errors) {
  print start_webfront("Confirmation","admin",3,-1);
  print CGI::h1((($#errors > 0) ? "Errors have" : "An error has")." been detected");
  print CGI::ul(CGI::li(\@errors));
  print CGI::p(CGI::i('Go <A HREF="userlist.cgi">back</A> and try again.'));
  print end_webfront();
  exit(1);
} 

# Do regular processing

if ($newpass ne '') {
  # Generate a random salt string for encryption
  my @chars = ('a'..'z', 'A'..'Z', '.', '/');
  my $salt = $chars[rand $#chars] . $chars[rand $#chars];
  # Encrypt the password and substitute it
  $user[1] = crypt $newpass, $salt;
}
if ( $q->param('newuser') eq 1 )
{
  $user[0] = $login;
  $user[2] = $q->param('uid');
  $user[7] = '/home/'. "$user[0]";
}
my %system_groups = read_group();
my $groupline = $system_groups{ 'users' };
$user[3] = $$groupline[2];
$user[5] = $name;
$user[8] = $q->param('shell');
my $replacement = join(":", @user[0,1,2,3,5,7,8]) . "\n";

# Now scan/replace password file entry
umask(022);
open IN, "/etc/passwd" or die "Can't read passwd\n";
open OUT, ">/etc/passwd.new" or die "Can't write passwd\n";
while (my $line = <IN>) {
  my @line = split /:/, $line;
  # Only change lines with matching login, and UID>100
  if ($line[0] eq $user[0] and $line[2] >= 100) {
    print OUT $replacement;
  } else {
    print OUT $line;
  }
}

############## add users home directory to /etc/passwd 

my $username;
my $homedir;

if ( $q->param('newuser') eq 1 )
{
  if ( $user[0] =~ /^([-a-zA-Z0-9]*)$/ )
  {
    $username = $1 ;
  }
  else
  {
    die "Users login name contains invalid characters: $user[0]" ;
  }
  print OUT $replacement;
  if ( $user[7] =~ /^(\/\bhome\b\/[-a-zA-Z0-9]*)$/ )
  {
    $homedir = $1 ;
  }
  else
  {
    die "Illegal characters in $user[7]" ;
  }
}

close OUT;
close IN;
system "mv /etc/passwd.new /etc/passwd";

################## add and update web server password file ###############

system "/usr/bin/htpasswd.sh";

##################### create users home and html directories ############
  
if ( $q->param('newuser') eq 1 )
{
  mkdir($homedir,0755);
  system( "chown $username:users $homedir" );
  mkdir ("$homedir/html",0755);
  system( "chown $username:users $homedir/html" );
  system( "cp /home/httpd/html/corel/blank.html $homedir/html/index.html" );
  system ("chown $username:users $homedir/html/index.html");
}

############################ Confirmation ##############################

print start_webfront("Confirmation","admin",3,-1);
print CGI::h1("Operation was successful.");
print CGI::p('Click <A HREF="userlist.cgi">here</A> to return.');
print end_webfront();
