#!/usr/bin/perl -w
#
# userlist.cgi
#
# Display a list of users on the system, with links to their homepages.
#
# Copyright 1998 Corel Computer Corporation
#
# This is open-source software (see www.opensource.org).  Open Source(R)
# is a Registered Certification Mark of Software in the Public Interest.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as 
# published by the Free Software Foundation; either version 1,
# or, (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to the
# Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

use CGI;
use strict;
require 'layout.pl';
require 'common.pl';

# Fetch parameters from common.pl script

my $host = host();
my %users = read_passwd();

# Determine name of publishing directory ("html" or "public_html")

my $dirname = publish_dir_name();

# Set up links for users who have a publishing directory

while (my ($key, $value) = each %users) {
  my $dir;
  if (-e "/home/$key/$dirname") {
    $dir = CGI::a({-href=>"http://$host/~$key/",
                   -target=>'_top'},'Home page');
  } else {
    $dir = CGI::i('None');
  }
  push @{$value}, $dir;		# Add it to the main hash-of-lists
 
  # Avoid white gaps in the output:
  if ($$value[4] eq '') { $$value[4] = '&nbsp;'; }
}

# Generate HTML table, sorted by login name

my $userlist;
for my $key (sort keys %users) {
  my $value = $users{$key};
  $userlist .= CGI::Tr({-align=>'center',-valign=>'middle',
 			-bgcolor=>'#E4E4E4'},
                       CGI::td($$value[0]) . 
                       CGI::td($$value[4]) . 
                       CGI::td($$value[-1]));
}

# Now print everything out

print start_webfront("List of Users","user",0,-1)."\n";

print CGI::table({-width=>'560',-border=>'0',-bgcolor=>'white'},
        CGI::Tr({-bgcolor=>'white'}, CGI::td({-colspan=>'3'},
          CGI::img({-src=>"/images/user/userdiaban.jpg"}))).
        CGI::Tr({-bgcolor=>'teal',-align=>'center'},
          join "\n", 
           map CGI::td(CGI::font({-face=>'Arial',-size=>'2',-color=>'white'},
            CGI::b{$_})), ('Login Name', 'Full Name', 'Services')).
        $userlist);

print end_webfront(); 
